class AudioPlayer {

	static get PANEL_HEIGHT() {return 100;};
	static get ID_LABEL_VOLUME() {return "AudioPlayerPanel-LabelVolume";};
	static get ID_LABEL_TRACK () {return "AudioPlayerPanel-LabelTrack" ;};
	static get BUTTON_LOOP    () {return "AudioPlayerPanel-ButtonLoop"   ;};
	static get BUTTON_LOOPALL () {return "AudioPlayerPanel-ButtonLoopAll";};
	static get BUTTON_SELECT  () {return "AudioPlayerPanel-ButtonSelect" ;};
	static get BUTTON_PLAY    () {return "AudioPlayerPanel-ButtonPlay"   ;};
	static get CLASS_PLAYING  () {return "audioPlayerPanel-playing";};
	static get CLASS_ON       () {return "audioPlayerPanel-on"     ;};

	constructor(files, options) {
		options = this._setAttributes(options);
		this._files      = files;
		this._lskeyName  = options.localStoragePathName;
		this._interval   = options.interval;
		this._sequential = options.sequential;
		this._loop       = false;
		this._loopAll    = false;
		this._selection  = false;
		this._volume     = 100;
		this._trackNo    = 1;
		this._trackNoSet = false;

		this._audio = this._makeAudio();
		this._panel = this._makePanel(options);
		this._init();
	}

	set loop(val) {
		val = (val ? true : false);
		this._loop =  val;
		this._setFunctionOn(`#${AudioPlayer.BUTTON_LOOP}`, val);
		if (val) {
			this._loopAll = false;
			this._setFunctionOn(`#${AudioPlayer.BUTTON_LOOPALL}`, false);
		}
	}

	get loop() {
		return this._loop;
	}

	set loopAll(val) {
		val = (val ? true : false);
		this._loopAll =  val;
		this._setFunctionOn(`#${AudioPlayer.BUTTON_LOOPALL}`, val);
		if (val) {
			this._loop = false;
			this._setFunctionOn(`#${AudioPlayer.BUTTON_LOOP}`, false);
		}
	}

	get loopAll() {
		return this._loopAll;
	}

	set selection(val) {
		val = (val ? true : false);
		this._selection =  val;
		this._setFunctionOn(`#${AudioPlayer.BUTTON_SELECT}`, val);
	}

	get selection() {
		return this._selection;
	}

	appendTo(element) {
		if (!element) {
			return;
		}
		element.appendChild(this._audio);
		element.appendChild(this._panel);
	}

	appendTrackListTo(element) {
		if (!element) {
			return;
		}
		let list = document.createElement("ul");
		let len = this._files.length;
		for (let i = 0; i < len; i++) {
			let item = document.createElement("li");
			item.textContent     = this._files[i].title;
			item.dataset.trackNo = i + 1;
			item.tabIndex        = 0;
			AppUtil.doLikeButton(item, e => {
				this.setTrackNo(item.dataset.trackNo);
				this.play();
			}, true);

			list.appendChild(item);
		}
		element.appendChild(list);
	}

	setTrackNo(trackNo) {
		if (trackNo == null || isNaN(trackNo)) {
			trackNo = 1;
		} else {
			trackNo = Number.parseInt(trackNo);
			if (trackNo < 1) {
				trackNo = this._files.length;
			} else if (trackNo > this._files.length) {
				trackNo = 1;
			}
		}
		this._trackNoSet = true;
		this._trackNo = trackNo;
		this._setTextContent(`#${AudioPlayer.ID_LABEL_TRACK} > *:last-child`, trackNo);
		return this._trackNo;
	}

	play() {
		if (this._trackNoSet) {
			this._audio.pause();
			this._audio.src = this._files[this._trackNo - 1].path;
			this._audio.load();
			this._audio.play();
			this._trackNoSet = false;
		} else {
			this._audio.play();
		}
		this._setPlayingTrack(true);
	}

	pause() {
		this._audio.pause();
		this._setPlayingTrack(false);
	}

	next() {
		this.setTrackNo(this._trackNo + 1);
		if (!this._audio.paused) {
			this.play();
		}
	}

	prev() {
		let paused = this._audio.paused;
		this._audio.pause();
		if (this._audio.currentTime > 1) {
			this._audio.currentTime = 0;
		} else {
			this.setTrackNo(this._trackNo - 1);
		}
		if (!paused) {
			this.play();
		}
	}

	turnUpVolume() {
		return this.setVolume(this._volume + 5);
	}

	turnDownVolume() {
		return this.setVolume(this._volume - 5);
	}

	setVolume(volume) {
		if (volume == null || isNaN(volume)) {
			volume = 0;
		} else {
			volume = Number.parseInt(volume);
			if (volume < 0) {
				volume = 0;
			} else if (volume > 100) {
				volume = 100;
			}
		}
		if (this._lskeyName) {
			this._setItemToLocalStorage(`volume@${this._lskeyName}`, volume);
		}
		this._audio.volume = volume * 0.01;
		this._volume = volume;
		this._setTextContent(`#${AudioPlayer.ID_LABEL_VOLUME} > *:last-child`, volume);
		return this._volume;
	}

	_setPlayingTrack(val) {
		let className = AudioPlayer.CLASS_PLAYING;
		val = (val ? true : false);
		if (val) {
			this._panel.classList.add(className);
		} else {
			this._panel.classList.remove(className);
		}
	}

	_playingTrack() {
		return this._panel.classList.contains(AudioPlayer.CLASS_PLAYING);
	}

	_setTextContent(selector, text) {
		this._panel.querySelector(selector).textContent = text;
	}

	_setFunctionOn(selector, on) {
		let element = this._panel.querySelector(selector);
		if (!element) {
			return;
		}
		if (on) {
			element.classList.add(AudioPlayer.CLASS_ON);
		} else {
			element.classList.remove(AudioPlayer.CLASS_ON);
		}
	}

	_init() {
		if (this._lskeyName) {
			let vol = this._getItemFromLocalStorage(`volume@${this._lskeyName}`);
			if (vol) {
				this.setVolume(vol);
			}
		}
		this.setTrackNo(1);
		this._audio.src = this._files[0].path;
		this._audio.load();
		this._audio.addEventListener("ended", e => {
			if (this._loop) {
				if (this._playingTrack()) {
					this.play();
				}
			} else if (this._sequential || this._loopAll) {
				let interval = this._files[this._trackNo - 1].interval;
				if (interval == null || isNaN(interval)) {
					interval = this._interval;
				}

				if (this._trackNo < this._files.length) {
					this.next();
				} else if (this._loopAll) {
					this.setTrackNo(1);
				} else {
					this._setPlayingTrack(false);
					return;
				}
				if (interval == 0) {
					this.play();
				} else {
					setTimeout(() => {
						if (this._playingTrack()) {
							this.play();
						}
					}, this._interval);
				}
			} else {
				this._setPlayingTrack(false);
			}
		});
	}

	_setItemToLocalStorage(key, val) {
		try {
			localStorage.setItem(key, val);
		} catch (e) {
		}
	}

	_getItemFromLocalStorage(key) {
		let ret = null;
		try {
			ret = localStorage.getItem(key);
		} catch (e) {
		}
		return ret;
	}

	_makeAudio() {
		let audio = document.createElement("audio");
		if (this._files && this._files.length) {
			audio.src = this._files[0].path;
		}
		audio.style = "display: none";
		return audio;
	}

	_makePanel(options) {
		let panel = this._makeSvgElement("svg");
		panel.appendChild(this._makeStyle(options.color, options.backgroundColor, options.hoveringColor));
		panel.appendChild(this._makeDefs());
		this._addParts(panel, options);
		return panel;
	}

	_addParts(svg, options) {
		let h = AudioPlayer.PANEL_HEIGHT;
		let w = 0;
		if (options.functions) {
			this._makeFuntionsPanel(svg, w, options.backgroundColor);
			w += 100;
		}
		if (options.controls) {
			this._makeControlsPanel(svg, w, options.backgroundColor);
			w += 200;
		}
		if (options.volume) {
			this._makeVolumePanel(svg, w, options.backgroundColor);
			w += 50;
		}

		svg.setAttribute("id", "AudioPlayerPanel");
		svg.setAttribute("width"  , w);
		svg.setAttribute("height" , h);
		svg.setAttribute("viewBox", `0 0 ${w} ${h}`);
		svg.setAttribute("version", "1.1");
	}

	_makeStyle(color, backgroundColor, hoveringColor) {
		let style = this._makeSvgElement("style");
		style.textContent = `
		/* <![CDATA[ */
		#AudioPlayerPanel {
			color: ${color};
			letter-spacing: normal;
		}
		text {
			font-family: "Arial Black", sans-serif;
			pointer-events: none;
		}
		#AudioPlayerPanel.audioPlayerPanel-playing g#${AudioPlayer.BUTTON_PLAY} > *:nth-child(2) {
			display:none;
		}
		#AudioPlayerPanel:not(.audioPlayerPanel-playing) > g#${AudioPlayer.BUTTON_PLAY} > *:nth-child(3) {
			display:none;
		}
		#AudioPlayerPanel > g.${AudioPlayer.CLASS_ON} use {
			fill: currentColor;
		}
		#AudioPlayerPanel > g.${AudioPlayer.CLASS_ON} text {
			fill: ${backgroundColor};
		}
		#AudioPlayerPanel > g:hover {
			cursor: pointer;
		}
		#AudioPlayerPanel > g:hover use:first-child {
			fill: ${hoveringColor};
		}
		#AudioPlayerPanel > g.${AudioPlayer.CLASS_ON}:hover use {
			fill: currentColor;
		}
		#AudioPlayerPanel > g.${AudioPlayer.CLASS_ON}:hover text {
			fill: ${hoveringColor};
		}
		/* ]]> */`;
		return style;
	}

	_makeDefs() {
		let defs = this._makeSvgElement("defs");
		defs.appendChild(this._makePath("AudioPlayerPanel-FunctionButton"   ,
		"M -28 -12 H  28 A  12 12 0 0 1 28  12 H -28 A 12 12 0 0 1 -28 -12 Z"));

		defs.appendChild(this._makePath("AudioPlayerPanel-VolumePlusButton" ,
		"M -12 0 V -18 A 12 12 0 0 1 12 -18 V 0"));

		defs.appendChild(this._makePath("AudioPlayerPanel-VolumeMinusButton",
		"M 12 0 V 18 A 12 12 0 0 1 -12 18 V 0"));

		defs.appendChild(this._makeCircle("AudioPlayerPanel-LargeButton", 32));
		defs.appendChild(this._makeCircle("AudioPlayerPanel-SmallButton", 22));

		let pause = this._makeGroupInDefs("AudioPlayerPanel-SymbolPause");
		pause.appendChild(this._makeRect(-11, -16, 8, 32));
		pause.appendChild(this._makeRect(  3, -16, 8, 32));
		defs.appendChild(pause);

		let play  = this._makeGroupInDefs("AudioPlayerPanel-SymbolPlay", "scale(0.4, 0.4)"     );
		play.appendChild(this._makePolyline("-14.6 -50, -14.6 50, 35.4 0, -14.6 -50"));
		defs.appendChild(play);

		let prev  = this._makeGroupInDefs("AudioPlayerPanel-SymbolPrev", "scale(-0.275, 0.275)");
		prev.appendChild(this._makeRect(21, -50, 12, 100));
		prev.appendChild(this._makePolyline("-33 -50,-33 50,17 0, -33 -50"));
		defs.appendChild(prev);

		let next  = this._makeGroupInDefs("AudioPlayerPanel-SymbolNext", "scale(0.275, 0.275)" );
		next.appendChild(this._makeRect(21, -50, 12, 100));
		next.appendChild(this._makePolyline("-33 -50,-33 50,17 0, -33 -50"));
		defs.appendChild(next);

		return defs;
	}

	_makePath(id, d) {
		let p = this._makeSvgElement("path");
		p.setAttribute("id", id);
		p.setAttribute("d" , d );
		p.setAttribute("stroke-width", 2);
		p.setAttribute("stroke", "currentColor");
		return p;
	}

	_makeCircle(id, r) {
		let c = this._makeSvgElement("circle");
		c.setAttribute("id", id);
		c.setAttribute("r" , r );
		c.setAttribute("stroke-width", 2);
		c.setAttribute("stroke", "currentColor");
		return c;
	}

	_makeGroupInDefs(id, transform) {
		let g = this._makeSvgElement("g");
		g.setAttribute("id", id);
		g.setAttribute("stroke-width"  , 0);
		g.setAttribute("pointer-events", "none");
		if (transform) {
			g.setAttribute("transform", transform);
		}
		return g;
	}


	_makeRect(x, y, w, h) {
		let r = this._makeSvgElement("rect");
		r.setAttribute("x"     , x);
		r.setAttribute("y"     , y);
		r.setAttribute("width" , w);
		r.setAttribute("height", h);
		return r;
	}

	_makePolyline(points) {
		let p = this._makeSvgElement("polyline");
		p.setAttribute("points", points);
		return p;
	}

	_makeFuntionsPanel(svg, xOffset, bgColor) {
		let x = 50 + xOffset;
		let loop = this._makeSvgElement("g");
		loop.setAttribute("id", AudioPlayer.BUTTON_LOOP);
		loop.appendChild(this._makeUse (x, 18, "AudioPlayerPanel-FunctionButton", bgColor));
		loop.appendChild(this._makeText(x, 22, "middle", 12, "LOOP"));
		loop.addEventListener("click", e => {this.loop = !this.loop;});
		svg.appendChild(loop);

		let all = this._makeSvgElement("g");
		all.setAttribute("id", AudioPlayer.BUTTON_LOOPALL);
		all.appendChild(this._makeUse (x, 50, "AudioPlayerPanel-FunctionButton", bgColor));
		all.appendChild(this._makeText(x, 54, "middle", 12, "LOOP ALL"));
		all.addEventListener("click", e => {this.loopAll = !this.loopAll;});
		svg.appendChild(all);

		let select = this._makeSvgElement("g");
		select.setAttribute("id", AudioPlayer.BUTTON_SELECT);
		select.appendChild(this._makeUse (x, 82, "AudioPlayerPanel-FunctionButton", bgColor));
		select.appendChild(this._makeText(x, 86, "middle", 12, "SELECT"));
		select.addEventListener("click", e => {this.selection = !this.selection;});
		// svg.appendChild(select);
	}

	_makeControlsPanel(svg, xOffset, bgColor) {
		let track = this._makeSvgElement("g");
		track.setAttribute("id", AudioPlayer.ID_LABEL_TRACK);
		track.appendChild(this._makeText(190 + xOffset, 14, "end", 12, "TRACK"));
		track.appendChild(this._makeText(190 + xOffset, 32, "end", 17, "0"));
		svg.appendChild(track);

		let prev = this._makeSvgElement("g");
		prev.appendChild(this._makeUse ( 33 + xOffset, 72, "AudioPlayerPanel-SmallButton", bgColor));
		prev.appendChild(this._makeUse ( 33 + xOffset, 72, "AudioPlayerPanel-SymbolPrev" , "currentColor" ));
		prev.addEventListener("click", e => {this.prev();});
		svg.appendChild(prev);

		let next = this._makeSvgElement("g");
		next.appendChild(this._makeUse (167 + xOffset, 72, "AudioPlayerPanel-SmallButton", bgColor));
		next.appendChild(this._makeUse (167 + xOffset, 72, "AudioPlayerPanel-SymbolNext" , "currentColor" ));
		next.addEventListener("click", e => {this.next();});
		svg.appendChild(next);

		let play = this._makeSvgElement("g");
		play.setAttribute("id", AudioPlayer.BUTTON_PLAY);
		play.appendChild(this._makeUse (100 + xOffset, 62, "AudioPlayerPanel-LargeButton", bgColor));
		play.appendChild(this._makeUse (100 + xOffset, 62, "AudioPlayerPanel-SymbolPlay" , "currentColor" ));
		play.appendChild(this._makeUse (100 + xOffset, 62, "AudioPlayerPanel-SymbolPause", "currentColor" ));
		play.addEventListener("click", e => {
			if (this._playingTrack()) {
				this.pause();
			} else {
				this.play();
			}
		});
		svg.appendChild(play);
	}

	_makeVolumePanel(svg, xOffset, bgColor) {
		let vol = this._makeSvgElement("g");
		vol.setAttribute("id", AudioPlayer.ID_LABEL_VOLUME);
		vol.appendChild(this._makeText(38 + xOffset, 14, "end", 12, "VOL"));
		vol.appendChild(this._makeText(37 + xOffset, 95, "end", 12, "100"));
		svg.appendChild(vol);

		let plus = this._makeSvgElement("g");
		plus.appendChild(this._makeUse (25 + xOffset, 50, "AudioPlayerPanel-VolumePlusButton", bgColor));
		plus.appendChild(this._makeText(25 + xOffset, 38, "middle", 20, "+"));
		plus.addEventListener("click", e => {this.turnUpVolume();});
		svg.appendChild(plus);

		let minus = this._makeSvgElement("g");
		minus.appendChild(this._makeUse (25 + xOffset, 50, "AudioPlayerPanel-VolumeMinusButton", bgColor));
		minus.appendChild(this._makeText(25 + xOffset, 74, "middle", 20, "-"));
		minus.addEventListener("click", e => {this.turnDownVolume();});
		svg.appendChild(minus);
	}

	_makeUse(x, y, linkedId, backgroundColor) {
		let u = this._makeSvgElement("use");
		u.setAttribute("x", x);
		u.setAttribute("y", y);
		u.setAttribute("fill"      , backgroundColor);
		u.setAttributeNS("http://www.w3.org/1999/xlink", "href", `#${linkedId}`);
		return u;
	}

	_makeText(x, y, textAnchor, fontSize, text) {
		let t = this._makeSvgElement("text");
		t.setAttribute("x", x);
		t.setAttribute("y", y);
		t.setAttribute("text-anchor", textAnchor    );
		t.setAttribute("font-size"  , fontSize      );
		t.setAttribute("fill"       , "currentColor");
		t.textContent = text;
		return t;
	}

	_makeSvgElement(name) {
		return document.createElementNS("http://www.w3.org/2000/svg", name);
	}

	_setAttributes(options) {
		let attr = this._defaultAttributes();
		if (!options) {
			return attr;
		}

		if (options.color) {
			attr.color = options.color;
		}
		if (options.backgroundColor) {
			attr.backgroundColor = options.backgroundColor;
		}

		attr.sequential = (options.sequential ? true : false);
		if (options.interval != null && !isNaN(options.interval)) {
			attr.interval = Number.parseInt(options.interval);
		}

		if (options.localStoragePathName) {
			attr.localStoragePathName = options.localStoragePathName;
		}

		if ("functions" in options) {
			attr.functions = (options.functions ? true : false);
		}
		if ("controls" in options) {
			attr.controls = (options.controls ? true : false);
		}
		if ("volume" in options) {
			attr.volume = (options.volume ? true : false);
		}

		return attr;
	}

	_defaultAttributes() {
		return {
			color          : "green",
			backgroundColor: "white",
			hoveringColor  : "yellow",
			sequential     : false,
			interval       : 1000,
			localStoragePathName: null,
			functions: true,
			controls : true,
			volume   : true
		};
	}
}
