function Read-Settings($path) {
	$script:sox = Get-Property $path "sox"
	If (-Not(Test-Path $sox -PathType Leaf)) {
		Write-Host "Invalid SoX File: $sox`n"
		break
	}
}

function Convert-Pitch($targetDir, $fileNameRegex) {

	$highDir = "${targetDir}_ph"
	$lowDir  = "${targetDir}_pl"

	Get-ChildItem $targetDir -Recurse | Where-Object {$_.Name -match $fileNameRegex} | ForEach-Object {
		$dirs = $_.DirectoryName.Remove(0, $targetDir.Length)
		$hDir = Join-Path $highDir $dirs
		$lDir = Join-Path $lowDir  $dirs
		New-VoiceFile $_ $hDir "ph_" 1.06
		New-VoiceFile $_ $lDir "pl_" 0.94
	}
}

function New-VoiceFile($file, $outDir, $prefix, $speed) {
	$newFileName = $file.Name.Insert(0, $prefix)
	$newFilePath = Join-Path $outDir $newFileName
	$orgFilePath = $file.FullName
	mkdir $outDir -Force | Out-Null

	If (($file.Name -eq "m06_04_037.ogg") -Or ($file.Name -eq "m06_04_040.ogg") ) {
		$vOpt = "-v 0.98"
	}
	&$sox $vOpt "$orgFilePath" "$newFilePath" speed $speed
}

$s = Get-Date
Write-Host "Making files of high and low pitch voices."

$base = Split-Path $MyInvocation.MyCommand.Path -Parent | Split-Path -Parent
. "${base}\page\utils.ps1"

Read-Settings "${base}\settings.txt"
Assert-SizeRemaining $base 0.6

Convert-Pitch "${base}\sound\arisen" "hm_vo_.+\.ogg"
Convert-Pitch "${base}\sound\pawn"   "[fm]0[567]_0\d_\d{3}\.ogg"

$e = Get-Date
$sec = [Math]::Round(($e - $s).TotalSeconds, [MidpointRounding]::AwayFromZero)
Write-Host "Completed. ($sec sec.)`n"

trap {
	$Error[0]
	break
}
