# Anne Donut wrote this script by modifying the original script written by nururefu.
#
# ORIGINAL SCRIPT:
#     Copyright (c) 2019 nururefu
#     https://github.com/nururefu/DDON-BGM-DECRYPTOR/blob/master/LICENSE

function Read-Settings($path) {
	$script:ddon = Get-Property $path "ddon"
	$script:sngw = Get-Property $path "sngw"
	If (-Not(Test-Path $ddon -PathType Container)) {
		Write-Host "Invalid DDON Folder: $ddon`n"
		break
	} ElseIf (-Not(Test-Path $sngw -PathType Leaf)) {
		Write-Host "Invalid SNGW File: $sngw`n"
		break
	}
}

function Convert-SoundFiles($srcDir, $dstDir, $renaming) {

	Write-Host "Converting files in $srcDir"
	Get-ChildItem $srcDir -Recurse -Filter "*.sngw" | ForEach-Object {

		$dir = $_.DirectoryName.Remove(0, $srcDir.Length)
		$leaf = Split-Path $dir -Leaf
		If (($leaf -eq "wave") -Or ($leaf -eq "di")) {
			$dir = Split-Path $dir -Parent
		}

		$fileName = [System.IO.Path]::ChangeExtension($_.Name, ".ogg")
		$len = $fileName.Length
		switch ($renaming) {
			1 {
				If (($fileName -match "st\d{4}ev\d{3}_n\d{4}_\d\.ogg") `
						-Or ($fileName -match "ev\d{3}_n\d{4}_\d\.ogg")) {
					$fileName = $fileName.Insert($len - 5, "0")
				}
			}
			2 {
				If (($fileName -match "em\d{6}_vo_[a-z]+\d\.ogg")) {
					$fileName = $fileName.Insert($len - 5, "0")
				}
			}
			3 {
				If (($fileName -match "q\d{8}_n\d{4}_\d\.ogg")) {
					$fileName = $fileName.Insert($len - 5, "0")
				}
			}
			4 {
				If (($fileName -match "[fm]\d{2}_\d{2}_\d{2}.ogg")) {
					$fileName = $fileName.Insert($len - 6, "0")
				}
			}
			5 {
				If (($fileName -match "hm_vo_[fm]_\d{2}_simple_\d{2}\.ogg")) {
					$fileName = $fileName.Insert($len - 6, "0")
				}
			}
		}

		$newDir  = Join-Path $dstDir $dir
		$newFile = Join-Path $newDir $fileName
		mkdir $newDir -Force | Out-Null
		Copy-Item $_.FullName $newFile -Force | Out-Null
		&$sngw "$newFile" | Out-Null
	}
}

$s = Get-Date

$base = Split-Path $MyInvocation.MyCommand.Path -Parent | Split-Path -Parent
. "${base}\page\utils.ps1"

Read-Settings "${base}\settings.txt"
Assert-SizeRemaining $base 3.6

$target = Join-Path ${ddon} "Dragon's Dogma Online\nativePC\sound\stream"
$output = Join-Path ${base} "sound"

Convert-SoundFiles "${target}\bgm"             "${output}\bgm"        0
Convert-SoundFiles "${target}\sfx\id_start"    "${output}\etc"        0
Convert-SoundFiles "${target}\vo\jp\np"        "${output}\res"        0
Convert-SoundFiles "${target}\ev"              "${output}\event"      1
Convert-SoundFiles "${target}\note\jp\n"       "${output}\character"  2
Convert-SoundFiles "${target}\sfx\id_mandrake" "${output}\mandragora" 2
Convert-SoundFiles "${target}\quest\di\jp"     "${output}\quest"      3
Convert-SoundFiles "${target}\note\jp\pw"      "${output}\pawn"       4
Convert-SoundFiles "${target}\vo\jp\hm"        "${output}\arisen"     5

$e = Get-Date
$sec = [Math]::Round(($e - $s).TotalSeconds, [MidpointRounding]::AwayFromZero)
Write-Host "Completed. ($sec sec.)`n"

trap {
	$Error[0]
	break
}
