window.addEventListener("DOMContentLoaded", init);

let player;
function init() {
	player = new AudioPlayer(FILES, {
		localStoragePathName: LOCAL_STORAGE_PATH_NAME,
		sequential: PLAY_SEQUENTIALLY
	});
	let box = q("#Jukebox");
	player.appendTo(box);
	player.appendTrackListTo(q("#TrackList"));
	if (typeof FILES_PH !== "undefined") {
		let p = makePitchSelector();
		box.appendChild(p);
		q("g:nth-of-type(2)", p).classList.add(CLASS_ON);
	}
}

function makePitchSelector() {
	let svg = makeSvgElement("svg");
	svg.setAttribute("id", "PitchSelector");
	svg.setAttribute("width"  ,  "50");
	svg.setAttribute("height" , "100");
	svg.setAttribute("viewBox", "0 0 50 100");
	svg.setAttribute("version", "1.1");
	svg.appendChild(makeStyle("green", "white", "yellow"));
	svg.appendChild(makeDefs());
	svg.appendChild(makeText(25, 14, 12, "PITCH"));
	svg.appendChild(makePitchButton(25, 30, "PitchSelector-Button", "HIGH", FILES_PH));
	svg.appendChild(makePitchButton(25, 57, "PitchSelector-Button", "MID" , FILES   ));
	svg.appendChild(makePitchButton(25, 84, "PitchSelector-Button", "LOW" , FILES_PL));
	return svg;
}

function makeStyle(color, backgroundColor, hoveringColor) {
	let style = makeSvgElement("style");
	style.textContent = `
	/* <![CDATA[ */
	#PitchSelector {
		color: ${color};
		letter-spacing: normal;
	}
	#PitchSelector text {
		font-family: "Arial Black", sans-serif;
		pointer-events: none;
	}
	#PitchSelector > g.${CLASS_ON} use {
		fill: currentColor;
	}
	#PitchSelector > g.${CLASS_ON} text {
		fill: ${backgroundColor};
	}
	#PitchSelector > g:hover {
		cursor: pointer;
	}
	#PitchSelector > g:hover use {
		fill: ${hoveringColor};
	}
	#PitchSelector > g.${CLASS_ON}:hover use {
		fill: currentColor;
	}
	#PitchSelector > g.${CLASS_ON}:hover text {
		fill: ${hoveringColor};
	}
	/* ]]> */`;
	return style;
}

function makeDefs() {
	let defs = makeSvgElement("defs");
	defs.appendChild(makePath("PitchSelector-Button", "M -10 -10 H  10 A  10 10, 0, 0, 1, 10 10 H -10 A 10 10, 0, 0, 1, -10 -10 Z"));
	return defs;
}

function makePath(id, d) {
	let p = makeSvgElement("path");
	p.setAttribute("id", id);
	p.setAttribute("d" , d );
	p.setAttribute("stroke-width", 2);
	p.setAttribute("stroke", "currentColor");
	return p;
}

function makePitchButton(x, y, linkedId, text, files) {
	let g = makeSvgElement("g");
	g.appendChild(makeUse (x, y, linkedId, "white"));
	g.appendChild(makeText(x, y + 4, 10, text));
	g.addEventListener("click", e => {
		let button = e.currentTarget;
		if (button.classList.contains(CLASS_ON)) {
			return;
		}
		q(`#PitchSelector .${CLASS_ON}`).classList.remove(CLASS_ON);
		button.classList.add(CLASS_ON);
		player._files = files;
		player.setTrackNo(player._trackNo);
	});
	return g;
}

function makeUse(x, y, linkedId, backgroundColor) {
	let u = makeSvgElement("use");
	u.setAttribute("x", x);
	u.setAttribute("y", y);
	u.setAttribute("fill", backgroundColor);
	u.setAttributeNS("http://www.w3.org/1999/xlink", "href", `#${linkedId}`);
	return u;
}

function makeText(x, y, fontSize, text) {
	let t = makeSvgElement("text");
	t.setAttribute("x", x);
	t.setAttribute("y", y);
	t.setAttribute("text-anchor", "middle"      );
	t.setAttribute("font-size"  , fontSize      );
	t.setAttribute("fill"       , "currentColor");
	t.textContent = text;
	return t;
}

function makeSvgElement(name) {
	return document.createElementNS("http://www.w3.org/2000/svg", name);
}

const CLASS_ON = "pitchSelector-on";
