class AppUtil {

	static getElementById(id, node) {
		return document.getElementById(id);
	}

	static querySelector(selector, node) {
		return AppUtil._getQueryTargetNode(node).querySelector(selector);
	}

	static querySelectorAll(selector, node) {
		return AppUtil._getQueryTargetNode(node).querySelectorAll(selector);
	}

	static scroll(target, behavior) {
		let y = target.getBoundingClientRect().top + window.scrollY;
		if (behavior && !navigator.userAgent.match(/\bEdge\b/)) {
			window.scroll({top: y, behavior: behavior});
		} else {
			window.scroll(0, y);
		}
	}

	static doLikeButton(element, eventFunction, spaceKey) {
		element.addEventListener("click", e => {
			eventFunction(e);
		});
		element.addEventListener("keydown", e => {
			if (Key.isEnter(e) || (spaceKey && Key.isSpace(e))) {
				e.preventDefault();
				eventFunction(e);
			}
		});
	}

	static physicalLengthOf(val) {
		if (val == null) {
			return null;
		} else if (val == 0) {
			return 0;
		} else if (!isNaN(val)) {
			return Number(val);
		}
		let div = document.createElement("div");
		div.style = `display: block; position: fixed; bottom: 0; left: ${val}; background-color: transparent; width: 0; border-width: 0;`;
		let body = document.body;
		body.appendChild(div);
		let length = div.getBoundingClientRect().left;
		body.removeChild(div);
		return length;
	}

	static _getQueryTargetNode(node) {
		return node ? node : document;
	}
}

class StringUtil {

	static pad(target, character, figure, head) {

		let count = figure - target.length;
		if (count <= 0) {
			return target;
		}

		let paddedString = "";
		for (let i = 0; i < count; i++) {
			paddedString += character;
		}

		if (head) {
			return paddedString + target;
		} else {
			return target + paddedString;
		}
	}

	static sanitize(str) {
		let sanitized = "";
		if (str) {
			for (let i = 0; i < str.length; i++) {
				sanitized += StringUtil._raplaceChar(str[i]);
			}
		}
		return sanitized;
	}

	static _raplaceChar(char) {
		switch(char) {
		case "&" : return "&amp;";
		case "<" : return "&lt;";
		case ">" : return "&gt;";
		case "\"": return "&quot;";
		default  : return char;
		}
	}
}

class Key {

	static withShift(e) {
		return e.shiftKey;
	}

	static withCtrl(e) {
		return e.ctrlKey;
	}

	static withAlt(e) {
		return e.altKey;
	}

	static isBackspace(e) {
		return e.keyCode == 8;
	}

	static isTab(e) {
		return e.keyCode == 9;
	}

	static isEnter(e) {
		return e.keyCode == 13;
	}

	static isEsc(e) {
		return e.keyCode == 27;
	}

	static isSpace(e) {
		return e.keyCode == 32;
	}

	static isPageUp(e) {
		return e.keyCode == 33;
	}

	static isPageDown(e) {
		return e.keyCode == 34;
	}

	static isEnd(e) {
		return e.keyCode == 35;
	}

	static isHome(e) {
		return e.keyCode == 36;
	}

	static isLeft(e) {
		return e.keyCode == 37;
	}

	static isUp(e) {
		return e.keyCode == 38;
	}

	static isRight(e) {
		return e.keyCode == 39;
	}

	static isDown(e) {
		return e.keyCode == 40;
	}

	static isInsert(e) {
		return e.keyCode == 45;
	}

	static isDelete(e) {
		return e.keyCode == 46;
	}

	static is0(e) {
		return e.keyCode == 48;
	}

	static is1(e) {
		return e.keyCode == 49;
	}

	static is2(e) {
		return e.keyCode == 50;
	}

	static is3(e) {
		return e.keyCode == 51;
	}

	static is4(e) {
		return e.keyCode == 52;
	}

	static is5(e) {
		return e.keyCode == 53;
	}

	static is6(e) {
		return e.keyCode == 54;
	}

	static is7(e) {
		return e.keyCode == 55;
	}

	static is8(e) {
		return e.keyCode == 56;
	}

	static is9(e) {
		return e.keyCode == 57;
	}

	static isA(e) {
		return e.keyCode == 65;
	}

	static isB(e) {
		return e.keyCode == 66;
	}

	static isC(e) {
		return e.keyCode == 67;
	}

	static isD(e) {
		return e.keyCode == 68;
	}

	static isE(e) {
		return e.keyCode == 69;
	}

	static isF(e) {
		return e.keyCode == 70;
	}

	static isG(e) {
		return e.keyCode == 71;
	}

	static isH(e) {
		return e.keyCode == 72;
	}

	static isI(e) {
		return e.keyCode == 73;
	}

	static isJ(e) {
		return e.keyCode == 74;
	}

	static isK(e) {
		return e.keyCode == 75;
	}

	static isL(e) {
		return e.keyCode == 76;
	}

	static isM(e) {
		return e.keyCode == 77;
	}

	static isN(e) {
		return e.keyCode == 78;
	}

	static isO(e) {
		return e.keyCode == 79;
	}

	static isP(e) {
		return e.keyCode == 80;
	}

	static isQ(e) {
		return e.keyCode == 81;
	}

	static isR(e) {
		return e.keyCode == 82;
	}

	static isS(e) {
		return e.keyCode == 83;
	}

	static isT(e) {
		return e.keyCode == 84;
	}

	static isU(e) {
		return e.keyCode == 85;
	}

	static isV(e) {
		return e.keyCode == 86;
	}

	static isW(e) {
		return e.keyCode == 87;
	}

	static isX(e) {
		return e.keyCode == 88;
	}

	static isY(e) {
		return e.keyCode == 89;
	}

	static isZ(e) {
		return e.keyCode == 90;
	}

	static isComma(e) {
		return e.keyCode == 188;
	}

	static isPeriod(e) {
		return e.keyCode == 190;
	}

	static isSlash(e) {
		return e.keyCode == 191;
	}

	static isBracketL(e) {
		return e.keyCode == 219;
	}

	static isBackslash(e) {
		return e.keyCode == 220;
	}

	static isBracketR(e) {
		return e.keyCode == 221;
	}
}

const idis = AppUtil.getElementById;
const q    = AppUtil.querySelector;
const qall = AppUtil.querySelectorAll;
